// -*- mode: cpp; mode: fold -*-
// Description								/*{{{*/
// $Id: cachefile.h,v 1.5 2002/04/27 04:28:04 jgg Exp $
/* ######################################################################
   
   CacheFile - Simple wrapper class for opening, generating and whatnot
   
   This class implements a simple 2 line mechanism to open various sorts
   of caches. It can operate as root, as not root, show progress and so on,
   it transparently handles everything necessary.
   
   This means it can rebuild caches from the source list and instantiates
   and prepares the standard policy mechanism.
   
   ##################################################################### */
									/*}}}*/
#ifndef PKGLIB_CACHEFILE_H
#define PKGLIB_CACHEFILE_H


#include <apt-pkg/depcache.h>
#include <apt-pkg/acquire.h>
#include <apt-pkg/sourcelist.h>
#include <apt-pkg/indexfile.h>

class pkgPolicy;
class pkgCacheFile
{
 protected:
   
   DynamicMMap *Map;
   pkgCache *Cache;
   pkgDepCache *DCache;
   pkgCache::PkgFileIterator ExtraIndexFile;

   public:

   pkgPolicy *Policy;
      
   // We look pretty much exactly like a pointer to a dep cache
   inline operator pkgCache &() {return *Cache;};
   inline operator pkgCache *() {return Cache;};
   inline operator pkgDepCache &() {return *DCache;};
   inline operator pkgDepCache *() {return DCache;};
   inline pkgDepCache *operator ->() {return DCache;};
   inline pkgDepCache &operator *() {return *DCache;};
   inline pkgDepCache::StateCache &operator [](pkgCache::PkgIterator const &I) {return (*DCache)[I];};
   inline unsigned char &operator [](pkgCache::DepIterator const &I) {return (*DCache)[I];};

   bool BuildCaches(OpProgress &Progress,bool WithLock = true);
   bool MergeExtraIndex(string Extra, OpProgress &Progress,bool WithLock = true);
   void InvalidateExtraIndexFile ();

   bool Open(OpProgress &Progress,bool WithLock = true);
   void Close();

   // Same as
   //
   // - Close
   // - write ExtraIndex to a file
   // - put that file into sources.list
   // - Open
   //
   // but much faster.  It is undefined whether the effects are
   // cummulative: every call to AddExtraIndex might or might not use
   // the ExtraIndices passed in from previous calls.  This freedom is
   // used to balance performance against cache growth: you can call
   // AddExtraIndex as often as you want and the cache will not grow
   // out of bounds, but stale information does not need to be removed
   // every time either.
   //
   // However, only the package versions of the most recent call to
   // AddExtraIndex will be considered as candidates.
   //
   // ExtraIndex should be a string formatted like a Debian Packages
   // file.  If it contains Filename fields, they should have absolute
   // URLs.
   //
   // Note that this function might actually call Close and Open,
   // which means that all old cache iterators, the old DepCache, and
   // other things become invalid.
   //
   // In order to actually install any package described by
   // ExtraIndex, you need to add a suitable pkgIndexFile to your
   // pkgSourceList object.  You can create such a pkgIndexFile with
   // GetExtraIndexFile.  The returned pkgIndexFile becomes unusable
   // as soon as AddExtraIndex is called again.
   
   bool AddExtraIndex(string ExtraIndex,OpProgress &Progress,bool WithLock = true);
   pkgIndexFile *GetExtraIndexFile ();
   pkgCache::PkgFileIterator GetExtraPackageFile ();

   pkgCacheFile();
   ~pkgCacheFile();
};

#endif
